// An AnimationMaster Segment

// Written by Bernie Roehl, November 1996

package hash;

import java.io.*;
import java.util.*;

public class Segment {
        protected int version;
        protected int platformType;
        protected int segmentId;
        protected Float minx, miny, minz, maxx, maxy, maxz;
        protected Float pivotX, pivotY, pivotZ;
        protected int nsplines;
        protected Spline[] splines;

        public int getVersion() { return version; }
        public int getPlatformType() { return platformType; }
        public int getSegmentId() { return segmentId; }
        public float getPivotX() { return pivotX.floatValue(); }
        public float getPivotY() { return pivotY.floatValue(); }
        public float getPivotZ() { return pivotZ.floatValue(); }
	public int getNumberOfSplines() { return nsplines; }
        public Spline getSpline(int n) { return splines[n]; }

        public ControlPoint findControlPoint(int cpid) {
                for (int i = 0; i < nsplines; ++i) {
                        Spline s = splines[i];
                        for (int j = 0; j < s.getNumberOfControlPoints(); ++j) {
                                if (s.getControlPoint(j).getCPID() == cpid)
                                        return s.getControlPoint(j);
                        }
                }
                return null;
        }

        public void dump() {
                System.out.println("Version " + version);
                System.out.println("Platform type = " + platformType);
                System.out.println("Segment ID = " + segmentId);
                System.out.println("Min: " + minx + " " + miny + " " + minz);
                System.out.println("Max: " + maxx + " " + maxy + " " + maxz);
                System.out.println("Pivot: " + pivotX + " " + pivotY + " " + pivotZ);
                System.out.println("Number of splines: " + nsplines);
                for (int i = 0; i < nsplines; ++i) {
                        System.out.println("Spline #" + i);
                        splines[i].dump();
                }
        }

        public Segment(DataInputStream input)
                throws IOException, HashSyntaxException {
                StringTokenizer s;

                s = new StringTokenizer(input.readLine());
                if (s.countTokens() != 7)
                        throw new HashSyntaxException("missing fields in header line");
                if (s.nextToken().equals("SCUL") == false)
                        throw new HashSyntaxException("missing signature in first line");
                version = Integer.parseInt(s.nextToken());
                s.nextToken();  // skip zero
                platformType = Integer.parseInt(s.nextToken());

                segmentId = Integer.parseInt(input.readLine());

                s = new StringTokenizer(input.readLine());
                if (s.countTokens() != 6)
                        throw new HashSyntaxException("missing fields in segment bounds");
                minx = new Float(s.nextToken());
                miny = new Float(s.nextToken());
                minz = new Float(s.nextToken());
                maxx = new Float(s.nextToken());
                maxy = new Float(s.nextToken());
                maxz = new Float(s.nextToken());

                s = new StringTokenizer(input.readLine());
                if (s.countTokens() != 3)
                        throw new HashSyntaxException("missing fields in segment pivot line");
                pivotX = new Float(s.nextToken());
                pivotY = new Float(s.nextToken());
                pivotZ = new Float(s.nextToken());

                nsplines = Integer.parseInt(input.readLine());
                splines = new Spline[nsplines];
                for (int i = 0; i < nsplines; ++i)
                        splines[i] = new Spline(input);
        }
}
